import logging

from .conf import Config
from .tokens import validate_or_redirect

logger = logging.getLogger(__name__)


class OktaMiddleware:
    """
    Middleware to validate JWT tokens set by Okta for authentication.
    """

    def __init__(self, get_response):
        self.config = Config()
        self.get_response = get_response

    def __call__(self, request):
        logger.debug("Entering Okta Middleware")

        if self.is_public_url(request.path_info):
            # We don't need tokens for public url's so just do nothing
            return self.get_response(request)

        redirect_response = validate_or_redirect(self.config, request)

        if redirect_response:
            return redirect_response

        return self.get_response(request)

    def is_public_url(self, url):
        return any(public_url.match(url) for public_url in self.config.public_urls)



from functools import wraps

from .conf import Config
from .tokens import validate_or_redirect


def okta_login_required(view_func):
    @wraps(view_func)
    def _wrapped_view(request, *args, **kwargs):
        config = Config()
        response = validate_or_redirect(config, request)
        if response:
            return response
        else:
            return view_func(request, *args, **kwargs)

    return _wrapped_view



from unittest.mock import Mock, patch

import pytest
from django.test import Client
from okta_oauth2.tests.utils import build_id_token


def test_decorator_prevents_unauthenticated_access(client: Client):
    """ If we're not authenticated we should return a redirect to the login """
    response = client.get("/decorated/")
    assert response.status_code == 302
    assert response.url == "/accounts/login/"


@pytest.mark.django_db
def test_decorator_allows_access_to_valid_token(client: Client):
    """
    If we have a valid token then we should allow access to the view.
    """
    nonce = "123456"
    token = build_id_token(nonce=nonce)

    client.cookies.load({"okta-oauth-nonce": nonce})

    session = client.session
    session["tokens"] = {"id_token": token}
    session.save()

    with patch("okta_oauth2.tokens.TokenValidator._jwks", Mock(return_value="secret")):
        response = client.get("/decorated/")
        assert response.status_code == 200


@pytest.mark.django_db
def test_decorator_disallows_access_to_invalid_token(client: Client):
    """ When an invalid token is supplied the decorator should reject. """
    nonce = "123456"
    token = "notvalid"

    client.cookies.load({"okta-oauth-nonce": nonce})

    session = client.session
    session["tokens"] = {"id_token": token}
    session.save()

    with patch("okta_oauth2.tokens.TokenValidator._jwks", Mock(return_value="secret")):
        response = client.get("/decorated/")
        assert response.status_code == 302
        assert response.url == "/accounts/login/"



from django.urls import path

from . import views

urlpatterns = [
    path("login/", views.login, name="login"),
    path("oauth2/callback/", views.callback, name="callback"),
    path("logout/", views.logout, name="logout"),
]
